package cn.com.qmth.examcloud.config.center.controller;

import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.commons.logging.ExamCloudLog;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLogFactory;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.config.center.core.BootstrapSecurityManager;
import cn.com.qmth.examcloud.config.center.core.PropertiesLoader;
import cn.com.qmth.examcloud.web.bootstrap.BootstrapSecurityUtil;
import io.swagger.annotations.ApiOperation;

@RestController
@RequestMapping("configCenter")
public class ConfigCenterController {

	private static ExamCloudLog log = ExamCloudLogFactory.getLog("APP_STARTUP_LOGGER");

	@ApiOperation(value = "消息通知")
	@PostMapping("notice")
	public ResponseEntity<String> notice(@RequestParam(required = true) String active,
			@RequestParam(required = true) String appCode,
			@RequestParam(required = true) String startupCode,
			@RequestParam(required = true) String message) {

		String mainInfo = "notice(). active=" + active + "; appCode=" + appCode + "; startupCode="
				+ startupCode + "; message=" + message;
		log.debug(mainInfo);

		String configCenterActive = BootstrapSecurityManager.getInstance().getActive();
		if (!configCenterActive.equals(active)) {
			log.error("active is wrong");
		}

		return new ResponseEntity<String>("OK", HttpStatus.OK);
	}

	@ApiOperation(value = "启动请求")
	@PostMapping("startup")
	public ResponseEntity<String> startup(@RequestParam(required = true) String active,
			@RequestParam(required = true) String appCode,
			@RequestParam(required = true) String startupCode, HttpServletResponse response) {

		String mainInfo = "startup(). active=" + active + "; appCode=" + appCode + "; startupCode="
				+ startupCode;

		log.debug(mainInfo);

		String configCenterActive = BootstrapSecurityManager.getInstance().getActive();
		if (!configCenterActive.equals(active)) {
			log.error("active is wrong");
			return new ResponseEntity<>("active is wrong", HttpStatus.INTERNAL_SERVER_ERROR);
		}

		Map<String, String> map = null;
		try {
			map = PropertiesLoader.getProperties(appCode, active);
		} catch (Exception e) {
			log.error(e.getMessage());
			return new ResponseEntity<>(e.getMessage(), HttpStatus.INTERNAL_SERVER_ERROR);
		}

		String ret = JsonUtil.toJson(map);
		ret = BootstrapSecurityUtil.encrypt(ret, startupCode);

		String appId = map.get("examcloud.bootstrap.app.appId");
		String secretKey = map.get("examcloud.bootstrap.app.secretKey");

		response.setHeader("App-Id", BootstrapSecurityUtil.encrypt(appId, startupCode));
		response.setHeader("App-Code", appCode);
		response.setHeader("Secret-Key", BootstrapSecurityUtil.encrypt(secretKey, startupCode));

		return new ResponseEntity<>(ret, HttpStatus.OK);
	}

}
