package cn.com.qmth.examcloud.config.center.core;

import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.lang3.StringUtils;

import cn.com.qmth.examcloud.commons.logging.ExamCloudLog;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLogFactory;
import cn.com.qmth.examcloud.commons.util.MapUtil;
import cn.com.qmth.examcloud.commons.util.RegExpUtil;
import cn.com.qmth.examcloud.commons.util.StringUtil;

/**
 * 命令解释器
 *
 * @author WANGWEI
 * @date 2018年12月5日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public class CommandInterpreter {

	private static ExamCloudLog log = ExamCloudLogFactory.getLog("COMMAND_LOGGER");

	private static CommandInterpreter interpreter;

	private static final Object LOCK = new Object();

	private static long sessionTimeout = 1000 * 60 * 5;

	/**
	 * 构造函数
	 */
	private CommandInterpreter() {
	}

	/**
	 * 获取单例
	 *
	 * @author WANGWEI
	 * @return
	 */
	public static CommandInterpreter getInstance() {
		if (null == interpreter) {
			synchronized (LOCK) {
				if (null == interpreter) {
					interpreter = new CommandInterpreter();
				}
				return interpreter;
			}
		} else {
			return interpreter;
		}
	}

	/**
	 * 解释
	 *
	 * @author WANGWEI
	 */
	public void interpret(String cmd) {
		try {
			process(cmd);
		} catch (Exception e) {
			log.error(e.getMessage());
		}
	}

	/**
	 * 处理
	 *
	 * @author WANGWEI
	 */
	public void process(String cmd) {
		cmd = cmd.trim();
		if (StringUtils.isBlank(cmd)) {
			return;
		}

		// 设置进程退出:session 10
		if (cmd.matches("\\s*session\\s+[1-9]\\d*")) {
			String timeout = RegExpUtil.find(cmd, "\\s*session\\s+([1-9]\\d*)", 1);
			timeout = timeout.length() > 4 ? "9999" : timeout;
			long timeoutLong = StringUtil.toLong(timeout);
			timeoutLong = timeoutLong > 60 ? 60 : timeoutLong;
			log.info("sessionTimeout = " + timeoutLong + " minutes");
			sessionTimeout = timeoutLong * 1000 * 60;
		}

		// 加载配置:load <appSimpleName>
		else if (cmd.matches("\\s*load\\s+([^\\s]+)\\s*")) {
			String appSimpleName = RegExpUtil.find(cmd, "\\s*load\\s+([^\\s]+)\\s*", 1);
			String active = BootstrapSecurityManager.getInstance().getActive();
			Map<String, String> properties = PropertiesLoader.getProperties(appSimpleName, active);

			properties = MapUtil.sortMapByKey(properties, true);
			StringBuilder sb = new StringBuilder("properties about ").append(appSimpleName)
					.append(" [").append(active).append("]:");
			for (Entry<String, String> entry : properties.entrySet()) {
				sb.append("\n   ").append(entry.getKey()).append(" : ").append(entry.getValue());
			}
			log.info(sb.toString());
		}

		// 错误命令
		else {
			log.error("error command !");
		}

	}

	public long getSessionTimeout() {
		return sessionTimeout;
	}

}
